@@ -62,9 +62,13 @@ npcdensbw.conbandwidth <-
   function(xdat = stop("data 'xdat' missing"),
            ydat = stop("data 'ydat' missing"),
            bws, bandwidth.compute = TRUE,
-           auto = TRUE,
-           nmulti, remin = TRUE, itmax = 10000,
-           ftol=1.19209e-07, tol=1.49012e-08, small=2.22045e-16,
+           nmulti, remin = TRUE, itmax = 10000, 
+           ftol = 1.490116e-07, tol = 1.490116e-04, small = 1.490116e-05,
+           memfac = 500.0, lbc.dir = 0.5, dfc.dir = 3, cfac.dir = 2.5*(3.0-sqrt(5)), initc.dir = 1.0, 
+           lbd.dir = 0.1, hbd.dir = 1, dfac.dir = 0.25*(3.0-sqrt(5)), initd.dir = 1.0, 
+           lbc.init = 0.1, hbc.init = 2.0, cfac.init = 0.5, 
+           lbd.init = 0.1, hbd.init = 0.9, dfac.init = 0.375, 
+           scale.init.categorical.sample=FALSE,
            ...){
 
     ydat = toFrame(ydat)
@@ -133,6 +137,10 @@ npcdensbw.conbandwidth <-
 
     tbw <- bws
 
+    mysd <- EssDee(data.frame(xcon,ycon))
+    nconfac <- nrow^(-1.0/(2.0*bws$cxkerorder+bws$ncon))
+    ncatfac <- nrow^(-2.0/(2.0*bws$cxkerorder+bws$ncon))
+
     if (bandwidth.compute){
       myopti = list(num_obs_train = nrow,
         iMultistart = ifelse(nmulti==0,IMULTI_FALSE,IMULTI_TRUE),
@@ -153,11 +161,13 @@ npcdensbw.conbandwidth <-
         xkerneval = switch(bws$cxkertype,
           gaussian = CKER_GAUSS + bws$cxkerorder/2 - 1,
           epanechnikov = CKER_EPAN + bws$cxkerorder/2 - 1,
-          uniform = CKER_UNI),
+          uniform = CKER_UNI,
+          "truncated gaussian" = CKER_TGAUSS),
         ykerneval = switch(bws$cykertype,
           gaussian = CKER_GAUSS + bws$cykerorder/2 - 1,
           epanechnikov = CKER_EPAN + bws$cykerorder/2 - 1,
-          uniform = CKER_UNI),
+          uniform = CKER_UNI,
+          "truncated gaussian" = CKER_TGAUSS),
         uxkerneval = switch(bws$uxkertype,
           aitchisonaitken = UKER_AIT,
           liracine = UKER_LR),
@@ -169,7 +179,7 @@ npcdensbw.conbandwidth <-
           liracine = OKER_LR),
         oykerneval = switch(bws$oykertype,
           wangvanryzin = OKER_WANG,
-          liracine = OKER_LR),
+          liracine = OKER_NLR),
         ynuno = dim(yuno)[2],
         ynord = dim(yord)[2],
         yncon = dim(ycon)[2],
@@ -177,27 +187,36 @@ npcdensbw.conbandwidth <-
         xnord = dim(xord)[2],
         xncon = dim(xcon)[2],
         fast = FALSE,
-        auto = auto)
+        old.cdens = FALSE,
+        int_do_tree = ifelse(options('np.tree'), DO_TREE_YES, DO_TREE_NO),
+        scale.init.categorical.sample = scale.init.categorical.sample,
+        dfc.dir = dfc.dir)
       
-      myoptd = list(ftol=ftol, tol=tol, small=small)
+      myoptd = list(ftol=ftol, tol=tol, small=small, memfac = memfac,
+        lbc.dir = lbc.dir, cfac.dir = cfac.dir, initc.dir = initc.dir, 
+        lbd.dir = lbd.dir, hbd.dir = hbd.dir, dfac.dir = dfac.dir, initd.dir = initd.dir, 
+        lbc.init = lbc.init, hbc.init = hbc.init, cfac.init = cfac.init, 
+        lbd.init = lbd.init, hbd.init = hbd.init, dfac.init = dfac.init, 
+        nconfac = nconfac, ncatfac = ncatfac)
 
       if (bws$method != "normal-reference"){
         myout=
           .C("np_density_conditional_bw", as.double(yuno), as.double(yord), as.double(ycon),
              as.double(xuno), as.double(xord), as.double(xcon),
+             as.double(mysd),
              as.integer(myopti), as.double(myoptd), 
              bw = c(bws$xbw[bws$ixcon],bws$ybw[bws$iycon],
                bws$ybw[bws$iyuno],bws$ybw[bws$iyord],
                bws$xbw[bws$ixuno],bws$xbw[bws$ixord]),
-             fval = double(2),
-             PACKAGE="np" )[c("bw","fval")]
+             fval = double(2), fval.history = double(max(1,nmulti)),
+             PACKAGE="np" )[c("bw","fval","fval.history")]
       } else {
         nbw = double(yncol+xncol)
         gbw = bws$yncon+bws$xncon
         if (gbw > 0){
-          nbw[1:gbw] = (4/3)^0.2
+          nbw[1:gbw] = 1.059224
           if(!bws$scaling)
-            nbw[1:gbw]=nbw[1:gbw]*EssDee(data.frame(xcon,ycon))*nrow^(-1.0/(2.0*bws$cxkerorder+gbw))
+            nbw[1:gbw]=nbw[1:gbw]*mysd*nconfac
         }
         myout= list( bw = nbw, fval = c(NA,NA) )
       }
@@ -226,6 +245,7 @@ npcdensbw.conbandwidth <-
 
       tbw$fval = myout$fval[1]
       tbw$ifval = myout$fval[2]
+      tbw$fval.history <- myout$fval.history
     }
     
     ## bandwidth metadata
@@ -242,7 +262,7 @@ npcdensbw.conbandwidth <-
     myf <- if(tbw$scaling) bwf else sff
     
     if ((tbw$xnuno+tbw$ynuno) > 0){
-      dfactor <- nrow^(-2.0/(2.0*tbw$cxkerorder+tbw$ncon))
+      dfactor <- ncatfac
       dfactor <- list(x = dfactor, y = dfactor)
 
       tl <- list(x = tbw$xdati$iuno, y = tbw$ydati$iuno)
@@ -251,7 +271,7 @@ npcdensbw.conbandwidth <-
     }
 
     if ((tbw$xnord+tbw$ynord) > 0){
-      dfactor <- nrow^(-2.0/(2.0*tbw$cxkerorder+tbw$ncon))
+      dfactor <- ncatfac
       dfactor <- list(x = dfactor, y = dfactor)
 
       tl <- list(x = tbw$xdati$iord, y = tbw$ydati$iord)
@@ -261,7 +281,7 @@ npcdensbw.conbandwidth <-
 
       
     if (tbw$ncon > 0){
-      dfactor <- nrow^(-1.0/(2.0*tbw$cxkerorder+tbw$ncon))
+      dfactor <- nconfac
       dfactor <- list(x = EssDee(xcon)*dfactor, y = EssDee(ycon)*dfactor)
 
       tl <- list(x = tbw$xdati$icon, y = tbw$ydati$icon)
@@ -284,6 +304,7 @@ npcdensbw.conbandwidth <-
                         oykertype = tbw$oykertype,
                         fval = tbw$fval,
                         ifval = tbw$ifval,
+                        fval.history = tbw$fval.history,
                         nobs = tbw$nobs,
                         xdati = tbw$xdati,
                         ydati = tbw$ydati,      
@@ -292,6 +313,9 @@ npcdensbw.conbandwidth <-
                         sfactor = tbw$sfactor,
                         bandwidth = tbw$bandwidth,
                         rows.omit = rows.omit,
+                        nconfac = nconfac,
+                        ncatfac = ncatfac,
+                        sdev = mysd,
                         bandwidth.compute = bandwidth.compute)
            
     tbw
@@ -327,8 +351,13 @@ npcdensbw.default <-
            ydat = stop("data 'ydat' missing"),
            bws, 
            bandwidth.compute = TRUE,
-           auto, nmulti, remin, itmax,
-           ftol, tol, small,
+           nmulti, remin, itmax, 
+           ftol, tol, small,memfac,
+           lbc.dir, dfc.dir, cfac.dir,initc.dir, 
+           lbd.dir, hbd.dir, dfac.dir, initd.dir, 
+           lbc.init, hbc.init, cfac.init, 
+           lbd.init, hbd.init, dfac.init, 
+           scale.init.categorical.sample,
            ## dummy arguments for conbandwidth() function call
            bwmethod, bwscaling, bwtype,
            cxkertype, cxkerorder,
@@ -369,8 +398,13 @@ npcdensbw.default <-
     ## next grab dummies for actual bandwidth selection and perform call
 
     mc.names <- names(match.call(expand.dots = FALSE))
-    margs <- c("bandwidth.compute", "auto", "nmulti", "remin", "itmax", "ftol",
-               "tol", "small")
+    margs <- c("bandwidth.compute", "nmulti", "remin", "itmax", "ftol",
+               "tol", "small", "memfac",
+               "lbc.dir", "dfc.dir", "cfac.dir","initc.dir", 
+               "lbd.dir", "hbd.dir", "dfac.dir", "initd.dir", 
+               "lbc.init", "hbc.init", "cfac.init", 
+               "lbd.init", "hbd.init", "dfac.init", 
+               "scale.init.categorical.sample")
     m <- match(margs, mc.names, nomatch = 0)
     any.m <- any(m != 0)
 
