@@ -2,7 +2,7 @@ sibandwidth <-
   function(beta, h,
            method=c("ichimura","kleinspady"),
            bwtype = c("fixed","generalized_nn","adaptive_nn"),
-           ckertype = c("gaussian", "epanechnikov","uniform"), 
+           ckertype = c("gaussian","truncated gaussian","epanechnikov","uniform"), 
            ckerorder = c(2,4,6,8),
            fval = NA,
            ifval = NA,
@@ -14,7 +14,9 @@ sibandwidth <-
            ynames = character(1),
            sfactor = NA, bandwidth = NA,
            rows.omit = NA, bandwidth.compute = TRUE,
-           optim.method = "NA", ...){
+           optim.method = "NA",
+           only.optimize.beta = FALSE,
+           ...){
 
   ndim = length(beta)
   regtype = "lc"
@@ -32,6 +34,9 @@ sibandwidth <-
       stop("ckerorder must be one of ", paste(kord,collapse=" "))
   }
 
+  if (ckertype == "truncated gaussian" && ckerorder != 2)
+    warning("using truncated gaussian of order 2, higher orders not yet implemented")
+
   porder = switch( ckerorder/2, "Second-Order", "Fourth-Order", "Sixth-Order", "Eighth-Order" )
 
   sumNum <- sfactor
@@ -71,7 +76,8 @@ sibandwidth <-
     pckertype = switch(ckertype,
       gaussian = paste(porder,"Gaussian"),
       epanechnikov =  paste(porder,"Epanechnikov"),
-      uniform = "Uniform"),
+      uniform = "Uniform",
+      "truncated gaussian" = "Truncated Gaussian"),
     nobs = nobs,
     ndim = ndim,
     ncon = sum(xdati$icon),
@@ -96,6 +102,9 @@ sibandwidth <-
     list(ckertype = ckertype,
          pckertype = mybw$pckertype))
 
+  if(only.optimize.beta)
+    mybw$pmethod <- ifelse(only.optimize.beta, paste("Pilot (bandwidth) +",mybw$pmethod, "(beta)"), mybw$pmethod)
+  
   if(!bandwidth.compute)
     mybw$pmethod <- "Manual"
 
@@ -128,7 +137,7 @@ coef.sibandwidth <- function(object, ...) {
  return(tc)
 }
 plot.sibandwidth <- function(...) { npplot(...) }
-predict.sibandwidth <- function(...) { eval(npindex(...), env = parent.frame()) }
+predict.sibandwidth <- function(...) { eval(npindex(...), envir = parent.frame()) }
 
 summary.sibandwidth <- function(object, ...){
   cat("\nSingle Index Model",
